﻿package  
{
	import org.flixel.*;
	/**
	 * ...
	 * @author mattdev
	 */
	public class Enemy extends FlxSprite
	{
		
		[Embed(source="../assets/textures/zombie.png")] 
		protected var playerImage:Class;
		[Embed(source="../assets/textures/blood.png")] 
		protected var bloodImage:Class;
		
		protected var pathToPlayer:FlxPath = null;
		protected var timeSincePathUpdate: Number = 0.0;
		
		protected var ENEMY_RUN_SPEED:int = 60;
		
		public function Enemy(speedMult: Number) 
		{
			super();
			immovable = true;
			loadGraphic(playerImage, true);
			
			addAnimation("run", [0, 1, 2, 3, 2, 1], 10);
			this.play("run");
			
			ENEMY_RUN_SPEED *= speedMult;
			
			var playState:PlayState = FlxG.state as PlayState;
			playState.miniMap.addSprite(this, 0xffff0000, 1);
		}
		
		public function updatePath(): void
		{
			var playState:PlayState = FlxG.state as PlayState;
			var distToPlayer:Number = FlxU.getDistance(this.getMidpoint(), playState.player.getMidpoint());
			
			if ( distToPlayer > 10.0 )
			{			
				pathToPlayer = playState.world.levelLayer.findPath(this.getMidpoint(), playState.player.getMidpoint(), false);
				timeSincePathUpdate = 0.0;
			}
			else
			{
				pathToPlayer = null;
			}
		}
		
		public function damage(dmg:Number): void
		{
			this.health -= dmg;

			var playState:PlayState = FlxG.state as PlayState;
			
			var emitter:FlxEmitter = new FlxEmitter(x,y); //x and y of the emitter
			 
			emitter.particleDrag = new FlxPoint(100, 100);
			emitter.makeParticles(bloodImage, 1, 8);
			playState.defaultGroup.add(emitter);
			emitter.start(true, 0.7, 0.1, 0);
			
			if ( this.health <= 0.0 )
			{
				exists = false;
				playState.setCash(playState.cash + 3 * ((70 - playState.exitTimer) / 60.0) * 6.0 );
				this.destroy();
			}
		}
		
		
		public override function destroy(): void
		{
			super.destroy();
			
			var playState:PlayState = FlxG.state as PlayState;
			playState.miniMap.removeSprite(this);
			
			playState.enemiesGroup.remove(this, true);
			
		}
		
		public override function update():void
		{
			
			//angle = FlxU.getAngle(FlxG.mouse.getWorldPosition(), this.getMidpoint());
			timeSincePathUpdate += FlxG.elapsed;
			
			
			var playState:PlayState = FlxG.state as PlayState;
			var distToPlayer:Number = FlxU.getDistance(this.getMidpoint(), playState.player.getMidpoint());
			
			if ( distToPlayer < 10.0 )
			{
				this.stopFollowingPath(true);
				this.pathToPlayer = null;
				velocity.x = 0.0;
				velocity.y = 0.0;
				playState.player.modifyHealth( -0.1 * FlxG.elapsed * playState.level );
				
			}
			else
			{
				if ( pathSpeed <= 0.0 )
				{
					this.stopFollowingPath(true);
					this.pathToPlayer = null;
					velocity.x = 0.0;
					velocity.y = 0.0;
				}
				
				if ( timeSincePathUpdate >= 1.0 || pathToPlayer == null )
				{
					updatePath();
				}
				
				if ( pathToPlayer && pathToPlayer != this.path )
				{
					this.followPath(pathToPlayer, ENEMY_RUN_SPEED * (Math.random() + 0.5), PATH_FORWARD, true);
				}
			}
			
			super.update();
		
		}
		
		public override function drawDebug(Camera:FlxCamera = null): void
		{
			super.drawDebug(Camera);
			
			if ( pathToPlayer )
			{
				//pathToPlayer.drawDebug();
			}
		}
		
	}

}