﻿package
{
	import org.flixel.*;
 
	public class PlayState extends FlxState
	{
		public var world:World;
		public var player:Player;
		
		
		public var healthBar: StatusBar;
		public var foodBar: StatusBar;
		
		public var miniMap:MiniMap;
		
		public var bulletsGroup: FlxGroup;
		public var enemiesGroup: FlxGroup;
		public var shopsGroup: FlxGroup;
		public var exitsGroup: FlxGroup;
		
		protected var timeSinceEnemySpawn: Number = 0.0;
		protected var timeSinceShopSpawn: Number = 999.0;
		
		public var defaultGroup: FlxGroup;
		public var uiGroup:FlxGroup;
		public var shopUIGroup:FlxGroup;
		
		public var cash: Number = 20;
		public var cashText: FlxText;
		public var ammoText: FlxText;
		public var exitText: FlxText;
		public var foodLabel: FlxText;
		public var hpLabel: FlxText;
		
		public var paused: Boolean = false;
		public var leaveShop: Boolean = false;
		
		public var shopMenu: ShopMenu;
		public var ammo: int = 0;
		
		public var exitTimer: Number = 60.0;
		
		public var level: int;
		
		public var startText: FlxText;


		[Embed(source="../assets/music/escape.mp3")] 
		static public var ThemeMusic:Class;		
		
		[Embed(source="../assets/sfx/item.mp3")] 
		static public var exitAppearsSound:Class;		
		
		public function PlayState(lvl:int): void
		{
			level = lvl;
			
		}
		
		override public function create():void
		{
			defaultGroup = new FlxGroup();
			add(defaultGroup);
			uiGroup = new FlxGroup();
			add(uiGroup);
			shopUIGroup = new FlxGroup();
			add(shopUIGroup);
			
			world = new World();
			
			miniMap = new MiniMap(world, 320 - 80 - 1, 240 - 80 - 1);

			bulletsGroup = new FlxGroup();
			enemiesGroup = new FlxGroup();
			shopsGroup = new FlxGroup();
			exitsGroup = new FlxGroup();
			
			defaultGroup.add(bulletsGroup);
			defaultGroup.add(enemiesGroup);
			defaultGroup.add(shopsGroup);
			defaultGroup.add(exitsGroup);
			
			player = new Player();
			player.x = player.y = 16;
			placeSpriteRandomly(player);
			FlxG.worldBounds = world.levelLayer.getBounds();
			
			FlxG.camera.follow(player, FlxCamera.STYLE_TOPDOWN_TIGHT);
				
			defaultGroup.add(player);

			player.init();
			

			healthBar = new StatusBar(this, 10, 17, 70, 6, 0xffff0000);
			foodBar = new StatusBar(this, 10, 33, 70, 6, 0xff00ff00);
			
			cashText = new FlxText(FlxG.width - 110, 6, 100, "Cash: " + (int)(cash));
			cashText.scrollFactor.x = cashText.scrollFactor.y = 0.0;
			cashText.setFormat(null, 10, 0xFFFFFFFF, "right", 0xFF5A5964);
			uiGroup.add(cashText);

			ammo = 100;
			ammoText = new FlxText(FlxG.width - 110, 16, 100, "Ammo: " + ammo);
			ammoText.scrollFactor.x = ammoText.scrollFactor.y = 0.0;
			ammoText.setFormat(null, 10, 0xFFFFFFFF, "right", 0xFF5A5964);
			uiGroup.add(ammoText);

			hpLabel = new FlxText(10, 6, 100, "HP");
			hpLabel.scrollFactor.x = hpLabel.scrollFactor.y = 0.0;
			hpLabel.setFormat(null, 8, 0xFFFFFFFF, "left", 0xFF5A5964);
			uiGroup.add(hpLabel);
			
			foodLabel = new FlxText(10, 22, 100, "Food");
			foodLabel.scrollFactor.x = foodLabel.scrollFactor.y = 0.0;
			foodLabel.setFormat(null, 8, 0xFFFFFFFF, "left", 0xFF5A5964);
			uiGroup.add(foodLabel);		
			
			exitText = new FlxText(0, 10, FlxG.width, "Time until escape: " + int(exitTimer));
			exitText.scrollFactor.x = exitText.scrollFactor.y = 0.0;
			exitText.setFormat(null, 8, 0xFFFFFFFF, "center", 0xFF5A5964);
			uiGroup.add(exitText);
			exitText.flicker(1);
			
			FlxG.playMusic(ThemeMusic);
			
			startText = new FlxText(0, FlxG.height / 2.0, FlxG.width, "Level " + level);
			startText.scrollFactor.x = startText.scrollFactor.y = 0.0;
			startText.setFormat(null, 8, 0xFFFFFFFF, "center", 0xFF5A5964);
			uiGroup.add(startText);
			
			this.setAmmo(this.ammo + 400 / level );
		}
		
		public function setCash(newCash: Number): void
		{
			cash = newCash;
			cashText.text = "Cash: " + (int)(cash);
		}
		
		public function setAmmo(newAmmo: int): void
		{
			if ( newAmmo == 0 && ammo > 0 )
			{
				ammoText.color = 0xffff0000;
				ammoText.flicker(1);
			}
			if ( newAmmo > 0 && ammo == 0 )
				ammoText.color = 0xffffffff;
			ammo = newAmmo;
			ammoText.text = "Ammo: " + ammo;
		}
		
		public function laserHitEnemy(o1: FlxObject, o2: FlxObject): void
		{
			(o2 as Enemy).damage(0.1);
			(o1 as Laser).exists = false;
			(o1 as Laser).destroy();
			
		}

		public function playerEnterShop(o1: FlxObject, o2: FlxObject): void
		{
			shopMenu = new ShopMenu(o2 as Shop);
		}
		
		public function playerExits(o1: FlxObject, o2: FlxObject): void
		{
			paused = true;
			
			FlxG.fade(0xff000000, 2.5, onFadeToNewLevel);
		}
		
		private function onFadeToNewLevel():void
		{
			FlxG.switchState(new PlayState(level + 1));
		}
		
		
		override public function update(): void
		{
			if ( paused )
			{
				return;
			}
			
			if ( shopMenu )
			{
				if ( leaveShop )
				{
					shopMenu.shop.destroy();
					shopMenu.destroy();
					shopMenu = null;
					leaveShop = false;
				}
				else
				{
					shopMenu.update();
					return;
				}
			}
			
			super.update();
			FlxG.collide(player, world.levelLayer);
			
			FlxG.collide(bulletsGroup, enemiesGroup, laserHitEnemy);
			FlxG.overlap(player, shopsGroup, playerEnterShop);
			FlxG.overlap(player, exitsGroup, playerExits);
			
			
			healthBar.setProgress(player.health);
			foodBar.setProgress(player.food);
			
			miniMap.update();
			
			timeSinceEnemySpawn += FlxG.elapsed;
			if ( timeSinceEnemySpawn >= 0.4 * ((exitTimer+100) / 60.0 ) / ((level + 5) / 5) )
			{
				if ( enemiesGroup.length < 100 )
					spawnEnemy();
				else
				{
					var enemy:Enemy = FlxU.getRandom(enemiesGroup.members) as Enemy;
					if ( enemy && !enemy.onScreen() )
						placeEnemy(enemy);
				}
				
				timeSinceEnemySpawn = 0.0;
			}
			
			timeSinceShopSpawn += FlxG.elapsed;
			if ( timeSinceShopSpawn >= 20.0 )
			{
				if ( shopsGroup.length < 3 )
					spawnShop();
				else
				{
					var shop:Shop = FlxU.getRandom(shopsGroup.members) as Shop;
					if ( shop && !shop.onScreen() )
						placeSpriteRandomly(shop);
				}
				timeSinceShopSpawn = 0.0;
			}
			
			if ( exitsGroup.length == 0 )
			{
				exitTimer -= FlxG.elapsed;
				
				if ( exitTimer <= 0.0 )
				{
					exitTimer = 0.0;
					var exitPoint:ExitPoint = new ExitPoint(0, 0);
					
					var isValidPosition:Boolean = false;
					
					var x:int = 0;
					var y:int = 0;
					while ( !isValidPosition )
					{
						isValidPosition = true;
						x = (int)(Math.random() * world.mapWidth);
						y = (int)(Math.random() * world.mapWidth);

						var tileValue:uint = world.levelLayer.getTile(x, y);
						
						if ( tileValue != 0 )
						{
							isValidPosition = false;
						}
					}
					
					exitPoint.x = x * 16.0 - exitPoint.width * 0.5;
					exitPoint.y = y * 16.0 - exitPoint.health * 0.5;
					
					exitsGroup.add(exitPoint);
					
					FlxG.play(exitAppearsSound);
				}
				
				if( exitTimer > 0.0 )
					exitText.text = "Time until escape: " + (int(exitTimer));
				else
				{
					exitText.text = "Head for the exit!";
					exitText.flicker(10);
				}
			}
			
			if ( startText ) 
			{
				startText.alpha = startText.alpha - 0.2 * FlxG.elapsed;
				if ( startText.alpha <= 0.0 )
				{
					uiGroup.remove(startText, true);
					startText.destroy();
					startText = null;
				}
			}
			
			this.setCash(this.cash + 2.0 * FlxG.elapsed);
			
		}
		
		public function placeSpriteRandomly(shop:FlxSprite): void
		{
			var isValid:Boolean = false;
			var spawnPoint:FlxPoint = new FlxPoint();
			
			var attempts:int = 0;
			
			while ( isValid == false )
			{
				spawnPoint.x = Math.random() * world.mapWidth * 16.0;
				spawnPoint.y = Math.random() * world.mapWidth * 16.0;
			
				isValid = true;
				if ( spawnPoint.x <= 16.0 || spawnPoint.y <= 16.0 || spawnPoint.x >= (world.mapWidth - 1) * 16.0 || spawnPoint.y >= (world.mapWidth - 1) * 16.0 )
				{
					spawnPoint.x = Math.min( Math.max(spawnPoint.x, 16.0), (world.mapWidth - 1) * 16.0 );
					spawnPoint.y = Math.min( Math.max(spawnPoint.y, 16.0), (world.mapWidth - 1) * 16.0 );					
					isValid = false;
				}
				
				shop.x = spawnPoint.x;
				shop.y = spawnPoint.y;

				if ( world.levelLayer.overlaps(shop) )
				{
					isValid = false;	
				}
				
				attempts++;
				
				if ( attempts > 100 )
					return;
			}
		}
		
		
		public function spawnShop(): void
		{
			var shop:Shop = new Shop(0, 0, ((Math.random() * 0.7) + 0.3) * ((70 - exitTimer) / 60.0) * 12.0 );
			
			placeSpriteRandomly(shop);
			
			shopsGroup.add(shop);
		}
		
		
		public function placeEnemy(enemy:Enemy): void
		{
			var isValid:Boolean = false;
			var spawnPoint:FlxPoint = null;
			
			var attempts:int = 0;
			while ( isValid == false )
			{
				spawnPoint = getRandomPointOutsideScreen();
			
				isValid = true;
				if ( spawnPoint.x <= 16.0 || spawnPoint.y <= 16.0 || spawnPoint.x >= (world.mapWidth - 1) * 16.0 || spawnPoint.y >= (world.mapWidth - 1) * 16.0 )
				{
					spawnPoint.x = Math.min( Math.max(spawnPoint.x, 16.0), (world.mapWidth - 1) * 16.0 );
					spawnPoint.y = Math.min( Math.max(spawnPoint.y, 16.0), (world.mapWidth - 1) * 16.0 );
					isValid = false;
				}
				
				enemy.x = spawnPoint.x;
				enemy.y = spawnPoint.y;

				if ( isValid && world.levelLayer.overlaps(enemy) )
				{
					isValid = false;	
				}
				
				attempts++;
				
				if ( attempts > 100 )
					return;
			}

		}
		
		public function spawnEnemy(): void
		{

			var enemy:Enemy = new Enemy(1.0 * (Math.random()+0.5) + (level-1) * 0.2);
			placeEnemy(enemy);
			
			enemiesGroup.add(enemy);
		}
		
		override public function draw(): void
		{
			super.draw();
			for ( var i:int = 0; i < enemiesGroup.length; i++ )
			{
				//enemies[i].drawDebug();
			}
		}
		
		public function getRandomPointOutsideScreen(): FlxPoint
		{
			// Find a point just outside of the visible area
			var tilesPerScreen:Number = 14.0;
			var xOffset:Number = (Math.random() - 0.5) * tilesPerScreen * 16.0;
			var yOffset:Number = (Math.random() - 0.5) * tilesPerScreen * 16.0;
			
			var edge: Number = Math.random();
			if ( edge < 0.25 )
			{
				// Top edge of screen
				yOffset = tilesPerScreen * 16.0;
			}
			else if( edge < 0.5 )
			{
				// Bottom edge of screen
				yOffset = -tilesPerScreen * 16.0;
			}
			else if( edge < 0.75 )
			{
				// Right edge of screen
				xOffset = tilesPerScreen * 16.0;
			}
			else
			{
				// Left edge of screen
				xOffset = -tilesPerScreen * 16.0;
			}
			
			return new FlxPoint(player.x + xOffset, player.y + yOffset);
		}
		
	}
}