﻿package  
{
	import org.flixel.*;
	/**
	 * ...
	 * @author mattdev
	 */
	public class ShopMenu extends FlxSprite
	{
		
		[Embed(source="../assets/textures/shopmenu.png")] 
		protected var shopImage:Class;
		[Embed(source="../assets/textures/shopfood.png")] 
		protected var foodImage:Class;
		[Embed(source="../assets/textures/shopammo.png")] 
		protected var ammoImage:Class;
		
		protected var greetingsText:FlxText;
		
		protected var foodSprite: FlxSprite;
		protected var ammoSprite: FlxSprite;

		protected var foodText:FlxText;
		protected var ammoText:FlxText;
		protected var leaveText:FlxText;
		
		public var shop:Shop;
		
		public function ShopMenu(shopEntering:Shop) 
		{
			super(0, 0, shopImage);
			
			shop = shopEntering;
			
			x = FlxG.width / 2.0 - this.width / 2.0;
			y = FlxG.height / 2.0- this.height / 2.0;
			
			this.scrollFactor.x = this.scrollFactor.y = 0.0;

			var playState:PlayState = FlxG.state as PlayState;
			
			playState.shopUIGroup.add(this);
			
			greetingsText = new FlxText(65, 65, 200, "Greetings, good sir. What would you like to purchase today?");
			greetingsText.scrollFactor.x = greetingsText.scrollFactor.y = 0.0;
			greetingsText.setFormat(null, 9, 0xFF000000, "center", 0xFFFFFFFF);
			playState.shopUIGroup.add(greetingsText);
			
			foodSprite = new FlxSprite(FlxG.width / 2.0 - (40 + 16), 110, foodImage);
			foodSprite.scrollFactor.x = foodSprite.scrollFactor.y = 0.0;
			playState.shopUIGroup.add(foodSprite);
			ammoSprite = new FlxSprite(FlxG.width / 2.0 + 40 - 16, 110, ammoImage);
			ammoSprite.scrollFactor.x = ammoSprite.scrollFactor.y = 0.0;
			playState.shopUIGroup.add(ammoSprite);
			
			var price: int = getPrice();
			
			foodText = new FlxText(foodSprite.x + 16 - 30, foodSprite.y + 30, 60, "Food: "+price);
			foodText.scrollFactor.x = foodText.scrollFactor.y = 0.0;
			foodText.setFormat(null, 8, 0xFF000000, "center");
			playState.shopUIGroup.add(foodText);			

			ammoText = new FlxText(ammoSprite.x + 16 - 30, ammoSprite.y + 30, 60, "Ammo: "+price);
			ammoText.scrollFactor.x = ammoText.scrollFactor.y = 0.0;
			ammoText.setFormat(null, 8, 0xFF000000, "center");
			playState.shopUIGroup.add(ammoText);	
			
			leaveText = new FlxText(0, 180, FlxG.width, "Leave store");
			leaveText.scrollFactor.x = leaveText.scrollFactor.y = 0.0;
			leaveText.setFormat(null, 8, 0xFF000000, "center");
			playState.shopUIGroup.add(leaveText);
		}
		
		
		public function getPrice(): int
		{
			return (int)(30.0 * shop.priceModifier);
		}
		
		override public function destroy():void
		{
			super.destroy();
			
			var playState:PlayState = FlxG.state as PlayState;
			playState.shopUIGroup.clear();
				
			greetingsText.destroy();
			
			foodSprite.destroy();
			ammoSprite.destroy();

			foodText.destroy();
			ammoText.destroy();
			leaveText.destroy();			
		}
		
		public override function update(): void
		{
			var selectionColor: uint = 0xFFFF0000;
			var greyedOutColor:uint = 0xFFDADADA;
			var mouseOverAmmo:Boolean = false;
			var mouseOverFood:Boolean = false;
			var mouseOverLeave:Boolean = false;
			var playState:PlayState = FlxG.state as PlayState;
			
			var price: int = getPrice();
			var canBuyFood:Boolean = playState.cash >= price && playState.player.food < 1.0 ;
			var canBuyAmmo:Boolean = playState.cash >= price; // && playState.player.food < 1.0 ;
			
			
			if ( ammoSprite.overlapsPoint(FlxG.mouse.getScreenPosition(), false) || ammoText.overlapsPoint(FlxG.mouse.getScreenPosition(), false) )
				mouseOverAmmo = true;
			if ( foodSprite.overlapsPoint(FlxG.mouse.getScreenPosition(), false) || foodText.overlapsPoint(FlxG.mouse.getScreenPosition(), false) )
				mouseOverFood = true;
			if ( leaveText.overlapsPoint(FlxG.mouse.getScreenPosition(), false) )
				mouseOverLeave = true;
								
			if( mouseOverAmmo )
				ammoText.color = selectionColor;
			else
				ammoText.color = 0xFF000000;				
			
			if ( !canBuyAmmo )
				ammoText.color = greyedOutColor;
				
			if ( mouseOverFood )
				foodText.color = selectionColor;
			else
				foodText.color = 0xFF000000;
				
			if ( !canBuyFood )
				foodText.color = greyedOutColor;
			
			if ( mouseOverLeave )
				leaveText.color = selectionColor;
			else
				leaveText.color = 0xFF000000;				
			
			if ( FlxG.mouse.justReleased() )
			{
				if ( mouseOverFood && canBuyFood )
				{
					playState.setCash(playState.cash - price);
					playState.player.modifyFood(0.8);
				}
				
				if ( mouseOverAmmo && canBuyAmmo )
				{
					playState.setCash(playState.cash - price);
					playState.setAmmo(playState.ammo + 100);
				}
				
				if ( mouseOverLeave )
				{
					playState.leaveShop = true;
				}
			}
		}
		
	}

}