﻿package  
{
	import org.flixel.*;
 
	/**
	 * ...
	 * @author mattdev
	 */
	public class World
	{
		public var mapWidth: int;
		
		public var groundLayer:FlxTilemap;
		public var roadLayer:FlxTilemap;
		public var levelLayer:FlxTilemap;
		
		public var data:Array = new Array();
		
		[Embed(source="../assets/textures/ground.png")] 
		protected var groundImage:Class;
		
		[Embed(source="../assets/textures/roads.png")] 
		protected var roadImage:Class;
		
		[Embed(source="../assets/textures/tiles.png")] 
		protected var tilesImage:Class;
		
		public var roadData: Array;
		public var groundData: Array;
		public var levelData: Array;
		
		public var shops: Array;
		
		
		public function World()
		{
			shops = new Array();
			
			mapWidth = 80;
			
			roadData = generateRoad();
			groundData = generateGround(roadData);
			levelData = generateLevel();
			
			groundLayer = new FlxTilemap();
			groundLayer.loadMap(FlxTilemap.arrayToCSV(groundData, mapWidth), groundImage, 16, 16, FlxTilemap.OFF, 0, 0);

			roadLayer = new FlxTilemap();
			roadLayer.loadMap(FlxTilemap.arrayToCSV(roadData, mapWidth), roadImage, 16, 16, FlxTilemap.AUTO);
			
			levelLayer = new FlxTilemap();
			levelLayer.loadMap(FlxTilemap.arrayToCSV(levelData, mapWidth), tilesImage, 16, 16, FlxTilemap.AUTO);
			
			
			var playState:PlayState = FlxG.state as PlayState;
			playState.defaultGroup.add(groundLayer);
			playState.defaultGroup.add(roadLayer);
			playState.defaultGroup.add(levelLayer);
			

			
		}

		public function generateRoad(): Array
		{
			var data:Array = new Array();
			
			var i:uint = 0;
			var j:uint = 0;
			
			for ( i = 0; i < mapWidth; i++ )
			{
				for ( j = 0; j < mapWidth; j++ )
				{
					var tileIndex: uint = i + j * mapWidth;
					
					data[tileIndex] = 0;
					
					if ( i % 10 == 1 || j % 10 == 1 )
					{
						data[tileIndex] = 1;
					}
				}
			}
			
			return data;
		}
		
		public function generateGround(roadData: Array): Array
		{
			var data:Array = new Array();
			
			var i:uint = 0;
			var j:uint = 0;
			
			for ( i = 0; i < mapWidth; i++ )
			{
				for ( j = 0; j < mapWidth; j++ )
				{
					var tileIndex: uint = i + j * mapWidth;
					
					data[tileIndex] = 0;
					if ( roadData[tileIndex] == 0 )
					{
						if ( Math.random() > 0.95 )
						{
							data[tileIndex] = 3;
						}
					}
				}
			}
			
			return data;
		}
		
		public function generateLevel(): Array
		{
			var data:Array = new Array();
			
			var i:uint = 0;
			var j:uint = 0;
			var k:uint = 0;
			
			for ( i = 0; i < mapWidth; i++ )
			{
				for ( j = 0; j < mapWidth; j++ )
				{
					var tileIndex: uint = i + j * mapWidth;
					
					data[tileIndex] = 1;
					
					if ( i % 10 <= 2 || j % 10 <= 2 )
					{
						data[tileIndex] = 0;
					}
					
					if ( i == 0 || i == mapWidth - 1 || j == 0 || j == mapWidth - 1 )
					{
						data[tileIndex] = 1;
					}

				}
			}
			
			var randLength:int = 10;
			
			for ( i = 0; i < mapWidth; i++ )
			{
				for ( j = 0; j < mapWidth; j++ )
				{
					tileIndex = i + j * mapWidth;
					
					if ( Math.random() > 0.98 )
					{
						var dir:int = (int)(Math.random() * 2);
						for ( k = 0; k < Math.random() * randLength; k++ )
						{
							data[tileIndex + k * (dir * mapWidth)] = 1;
						}
					}
					
					/*if ( Math.random() > 0.99 )
					{
						data[tileIndex] = 2;
						data[tileIndex+1] = 2;
						data[tileIndex+mapWidth] = 2;
						data[tileIndex+mapWidth+1] = 2;
					}*/
				}
			}
			
			return data;
		}
		
	}

}